
/*
 * $Xorg:xpapputil.h,v 1.1 2002/06/10 02:54:18 gisburn Exp $
 * 
 * xpapputil - Application level utility library for Xprint
 *
 * 
Copyright 2002 Roland Mainz <roland.mainz@informatik.med.uni-giessen.de>

All Rights Reserved.

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of The Open Group shall not be
used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from The Open Group.
 *
 * Author:  Roland Mainz <roland.mainz@informatik.med.uni-giessen.de>
 */

#ifndef XPAPPUTILS_H
#define XPAPPUTILS_H 1
 
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/XprintUtil/xprintutil.h>

#ifdef DEBUG
#define XPAU_RETURN_IF_FAIL(expr, return_code) \
    if( !(expr) ) { \
      printf("XPAU_RETURN_IF_FAIL(file %s, line %d): " #expr ", returning " #return_code "\n", __FILE__, __LINE__); \
      return (return_code); \
    }

#define XPAU_DEBUG(x) x

#else

#define XPAU_RETURN_IF_FAIL(expr, return_code) \
    if( !(expr) ) { \
      return (return_code); \
    }

#define XPAU_DEBUG(x)
#endif /* DEBUG */

/*
 * Struct for XpauGetContext(), XpauReleaseContext()
 * 
 */
typedef struct
{
    Display                 *pdpy;
    XPContext                pcontext;
    int                      xp_event_base,
                             xp_error_base;

    Bool                     inJob;
    Bool                     inDocument;
    Bool                     inPage;

    void                    *print_to_filehandle;
    Screen                  *pscreen;
    int                      pscreennumber;
    long                     document_dpi;
    
    XpuSupportedFlags        supported_job_attributes;
    XpuSupportedFlags        supported_doc_attributes;
    XpuSupportedFlags        supported_page_attributes;
    
    XpuMediumSourceSizeList  medium_list;
    int                      medium_num_list_entries;
    XpuResolutionList        resolution_list;
    int                      resolution_num_list_entries;
    XpuOrientationList       orientation_list;
    int                      orientation_num_list_entries;
    XpuPlexList              plex_list;
    int                      plex_num_list_entries;
} XpauContext;


#define XpauVFilename      (1L<<0)
#define XpauVMedium        (1L<<1)
#define XpauVResolution    (1L<<2)
#define XpauVOrientation   (1L<<3)
#define XpauVPlex          (1L<<4)
#define XpauVCopies        (1L<<5)
#define XpauVTitle         (1L<<6)
#define XpauVAll (XpauVFilename    | \
                  XpauVMedium      | \
                  XpauVResolution  | \
                  XpauVOrientation | \
                  XpauVPlex        | \
                  XpauVCopies      | \
                  XpauVTitle)

typedef unsigned long XpauFlags;

typedef struct
{
  const char              *title;
  long                     copies;
} XpauJobValues;

typedef struct
{
  XpuMediumSourceSizeRec  *medium;
  XpuResolutionRec        *resolution;
  XpuOrientationRec       *orientation;
  XpuPlexRec              *plex;
  long                     copies;
} XpauDocValues;

typedef struct
{
  XpuMediumSourceSizeRec  *medium;
  XpuResolutionRec        *resolution;
  XpuOrientationRec       *orientation;
  XpuPlexRec              *plex;
} XpauPageValues;

#define XPAUERROR_SUCCESS                        (0)
#define XPAUERROR_ERROR_NO_MEMORY               (-1)
#define XPAUERROR_UNEXPECTED_EOS                (-2)
#define XPAUERROR_UNKNOWN_ARGUMENT              (-3)
#define XPAUERROR_UNSUPPORTED_MEDIUM            (-4)
#define XPAUERROR_UNSUPPORTED_RESOLUTION        (-5)
#define XPAUERROR_UNSUPPORTED_ORIENTATION       (-7)
#define XPAUERROR_UNSUPPORTED_PLEX              (-8)
#define XPAUERROR_UNSUPPORTED_COPY_COUNT        (-9)
#define XPAUERROR_UNSUPPORTED_JOB_NAME         (-10)

/* Prototypes */
XpauContext *XpauGetContext( const char *printername );
void         XpauReleaseContext( XpauContext *context );
int XpauParseArgs( XpauContext *context,
                   XpauFlags *jobvaluemask, XpauJobValues *jobvalues,
                   XpauFlags *docvaluemask, XpauDocValues *docvalues,
                   char **jobfilename,
                   const char *argument_string );
int          XpauSetJobValues( XpauContext *context, XpauFlags valuemask, XpauJobValues *Xpaujobvalues );
int          XpauSetDocValues( XpauContext *context, XpauFlags valuemask, XpauDocValues *Xpaudocvalues );
int          XpauSetPageValues( XpauContext *context, XpauFlags valuemask, XpauPageValues *Xpaupagevalues );
Bool         XpauStartJob( XpauContext *context, const char *printerfile );
Bool         XpauEndJob(XpauContext *context);
void         XpauWaitForPrintNotify(XpauContext *context, int type);

Bool         XpauStartJob( XpauContext *context, const char *printerfile );
Bool         XpauEndJob( XpauContext *context );
Bool         XpauStartDocument( XpauContext *context, XPDocumentType type );
Bool         XpauEndDocument( XpauContext *context );
Bool         XpauStartPage( XpauContext *context, Window pwin );
Bool         XpauEndPage( XpauContext *context );

#endif /* !XPAPPUTILS_H */


